"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bfj_1 = __importDefault(require("bfj"));
const fs = __importStar(require("fs-extra"));
const gluegun_1 = require("gluegun");
const path = __importStar(require("path"));
const FileSizeReporter_1 = __importDefault(require("react-dev-utils/FileSizeReporter"));
const formatWebpackMessages_1 = __importDefault(require("react-dev-utils/formatWebpackMessages"));
const printHostingInstructions_1 = __importDefault(require("react-dev-utils/printHostingInstructions"));
const webpack_1 = __importDefault(require("webpack"));
const environment_1 = require("../../../environment");
const packager_1 = require("../../../packager");
const paths_1 = require("../../../paths");
const utils_1 = require("../../../utils");
const neutrino_1 = require("../../../webpack/neutrino");
const command_1 = require("../../command");
const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;
function copyPublicFolder() {
    gluegun_1.print.info(`Copying ${paths_1.PUBLIC_PATH} into ${paths_1.DIST_PATH}`);
    fs.copySync(paths_1.PUBLIC_PATH, paths_1.DIST_PATH, {
        dereference: true,
        filter: file => file !== paths_1.HTML_PATH,
    });
}
function measureFileSizesBeforeBuild() {
    gluegun_1.print.info('Measure file sizes');
    return FileSizeReporter_1.default.measureFileSizesBeforeBuild(paths_1.DIST_PATH);
}
async function emptyBuildFolder() {
    gluegun_1.print.info(`Emptying ${paths_1.DIST_PATH}`);
    // Remove all content but keep the directory so that
    // if you're in it, you don't end up in Trash
    fs.emptyDirSync(paths_1.DIST_PATH);
}
async function build(webpackConfig, opts) {
    const options = {
        writeStatsJson: true,
        ...opts,
    };
    const spinner = gluegun_1.print.spin('Compiling');
    const compiler = webpack_1.default(webpackConfig);
    return new Promise((resolve, reject) => {
        compiler.run((err, stats) => {
            let messages;
            if (err) {
                if (!err.message) {
                    spinner.fail('Compilation Failed');
                    reject(err);
                    return;
                }
                messages = formatWebpackMessages_1.default({
                    errors: [err.message],
                    warnings: [],
                });
            }
            else {
                messages = formatWebpackMessages_1.default(stats.toJson({ all: false, warnings: true, errors: true }));
            }
            if (messages.errors.length) {
                // Only keep the first error. Others are often indicative
                // of the same problem, but confuse the reader with noise.
                if (messages.errors.length > 1) {
                    messages.errors.length = 1;
                }
                spinner.fail('Compilation Failed');
                reject(new Error(messages.errors.join('\n\n')));
                return;
            }
            if (opts.isCI && messages.warnings.length) {
                spinner.fail('Compilation Failed');
                gluegun_1.print.info('\nTreating warnings as errors because process.env.CI = true.\n Most CI servers set it automatically.\n');
                reject(new Error(messages.warnings.join('\n\n')));
                return;
            }
            const resolveArgs = {
                stats,
                warnings: messages.warnings,
            };
            if (options.writeStatsJson) {
                bfj_1.default
                    .write(paths_1.STATS_PATH, stats.toJson())
                    .then(() => {
                    spinner.succeed('Compiled successfully 🥳');
                    resolve(resolveArgs);
                })
                    .catch(reject);
                return;
            }
            spinner.succeed('Compiled successfully 🥳');
            resolve(resolveArgs);
        });
    });
}
function printFileSizesAfterBuild(stats, previousFileSizes) {
    gluegun_1.print.info('\nFile sizes after gzip:\n');
    FileSizeReporter_1.default.printFileSizesAfterBuild(stats, previousFileSizes, paths_1.DIST_PATH, WARN_AFTER_BUNDLE_GZIP_SIZE, WARN_AFTER_CHUNK_GZIP_SIZE);
    console.log(''); // Do not remove it
}
function showHostingInstructions(webpackConfig) {
    const useYarn = environment_1.getPackageManager() === environment_1.PackageManagers.yarn;
    const appPackage = packager_1.readPackage(paths_1.CWD);
    const publicUrl = utils_1.withoutSlash(webpackConfig.output.publicPath);
    printHostingInstructions_1.default(appPackage, publicUrl, webpackConfig.output.publicPath, path.relative(paths_1.CWD, paths_1.DIST_PATH), useYarn);
}
function getPublicPath(opts) {
    const publicPath = (() => {
        if (opts.publicPath) {
            return opts.publicPath;
        }
        const pkg = packager_1.readPackage(paths_1.CWD);
        // We use "homepage" field to infer "public path" at which the app is served.
        // Webpack needs to know it to put the right <script> hrefs into HTML even in
        // single-page apps that may serve index.html for nested URLs like /todos/42.
        // We can't use a relative path in HTML because we don't want to load something
        // like /todos/42/static/js/bundle.7289d.js. We have to know the root.
        if (pkg.homepage) {
            return pkg.homepage;
        }
        return '/';
    })();
    return utils_1.withSlash(publicPath);
}
const buildCommand = command_1.createCommand({
    dashed: false,
    description: 'Build an application',
    env: environment_1.Environments.production,
    async run(toolbox) {
        const options = toolbox.parameters.options || {};
        const publicPath = getPublicPath(options);
        const config = neutrino_1.configFactory({
            env: environment_1.Environments.production,
            publicPath,
            sourceMapPublicPath: options.sourceMapPublicPath,
            strawHatConfig: toolbox.strawHat.config,
        });
        const webpackConfig = config.webpack();
        const previousFileSizes = await measureFileSizesBeforeBuild();
        await emptyBuildFolder();
        copyPublicFolder();
        const { stats } = await build(webpackConfig, {
            isCI: environment_1.isCI,
            writeStatsJson: options.stats,
        });
        printFileSizesAfterBuild(stats, previousFileSizes);
        showHostingInstructions(webpackConfig);
    },
});
exports.default = buildCommand;
//# sourceMappingURL=build.js.map