"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gluegun_1 = require("gluegun");
const ramda_1 = require("ramda");
const clearConsole_1 = __importDefault(require("react-dev-utils/clearConsole"));
const openBrowser_1 = __importDefault(require("react-dev-utils/openBrowser"));
const WebpackDevServerUtils_1 = require("react-dev-utils/WebpackDevServerUtils");
const webpack_1 = __importDefault(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const environment_1 = require("../../../environment");
const packager_1 = require("../../../packager");
const paths_1 = require("../../../paths");
const utils_1 = require("../../../utils");
const devServerConfig_1 = require("../../../webpack/devServerConfig");
const neutrino_1 = require("../../../webpack/neutrino");
const command_1 = require("../../command");
const getWebpackDevServerOverride = ramda_1.pathOr(undefined, [
    'app',
    'devServerOverride',
]);
const startCommand = command_1.createCommand({
    dashed: false,
    description: 'Start local dev-server',
    env: environment_1.Environments.development,
    async run(toolbox) {
        const options = {
            dangerouslyDisableHostCheck: false,
            host: '0.0.0.0',
            https: false,
            openBrowser: true,
            port: '5000',
            ...toolbox.parameters.options,
        };
        const defaultPort = parseInt(options.port, 10);
        const protocol = options.https ? 'https' : 'http';
        const isInteractive = process.stdout.isTTY;
        const pkg = packager_1.readPackage(paths_1.CWD);
        const useYarn = environment_1.getPackageManager() === environment_1.PackageManagers.yarn;
        const port = await WebpackDevServerUtils_1.choosePort(options.host, defaultPort);
        if (port === undefined) {
            gluegun_1.print.error('Can not find a port');
            process.exit(1);
        }
        const urls = WebpackDevServerUtils_1.prepareUrls(protocol, options.host, port);
        const publicPath = utils_1.withSlash(urls.localUrlForBrowser);
        const config = neutrino_1.configFactory({
            env: environment_1.Environments.development,
            publicPath,
            strawHatConfig: toolbox.strawHat.config,
        });
        const webpackConfig = config.webpack();
        const compiler = WebpackDevServerUtils_1.createCompiler(webpack_1.default, webpackConfig, pkg.name, urls, useYarn);
        const proxyConfig = WebpackDevServerUtils_1.prepareProxy(toolbox.strawHat.config.proxy, paths_1.PUBLIC_PATH);
        // Serve webpack assets generated by the compiler over a web server.
        const serverConfig = devServerConfig_1.devServerConfigFactory({
            allowedHost: urls.lanUrlForConfig,
            dangerouslyDisableHostCheck: options.dangerouslyDisableHostCheck,
            devServerOverride: getWebpackDevServerOverride(toolbox.strawHat.config),
            host: options.host,
            protocol,
            proxy: proxyConfig,
            publicPath,
        });
        const devServer = new webpack_dev_server_1.default(compiler, serverConfig);
        // Launch WebpackDevServer.
        devServer.listen(port, options.host, err => {
            if (err) {
                gluegun_1.print.error(err);
                return;
            }
            if (isInteractive) {
                clearConsole_1.default();
            }
            gluegun_1.print.info('Starting the development server...\n');
            if (options.openBrowser) {
                openBrowser_1.default(urls.localUrlForBrowser);
            }
        });
        ['SIGINT', 'SIGTERM'].forEach((sig) => {
            process.on(sig, () => {
                devServer.close();
                process.exit();
            });
        });
    },
});
exports.default = startCommand;
//# sourceMappingURL=start.js.map