"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const dotenv_1 = __importDefault(require("dotenv"));
const dotenv_expand_1 = __importDefault(require("dotenv-expand"));
const gluegun_1 = require("gluegun");
const path = __importStar(require("path"));
const ramda_1 = require("ramda");
const paths_1 = require("./paths");
var PackageManagers;
(function (PackageManagers) {
    PackageManagers["npm"] = "npm";
    PackageManagers["yarn"] = "yarn";
})(PackageManagers = exports.PackageManagers || (exports.PackageManagers = {}));
var Environments;
(function (Environments) {
    Environments["development"] = "development";
    Environments["test"] = "test";
    Environments["production"] = "production";
})(Environments = exports.Environments || (exports.Environments = {}));
const ALLOWED_ENV = [
    Environments.development,
    Environments.test,
    Environments.production,
];
exports.isCI = !!process.env.CI;
/**
 * Setup the current environment. Normalizing `NODE_PATH`, loading the `dotEnv`
 * files and setting up the correct environment variables.
 */
exports.setupEnv = ramda_1.compose(setNodePath, loadEnvVariables, setEnvMode);
/**
 * We support resolving modules according to `NODE_PATH`.
 * This lets you use absolute paths in imports inside large monorepo:
 * https://github.com/facebook/create-react-app/issues/253.
 *
 * It works similar to `NODE_PATH` in Node itself:
 * https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
 *
 * Note that unlike in Node, only *relative* paths from `NODE_PATH` are honored.
 * Otherwise, we risk importing Node.js core modules into an app instead of
 * Webpack shims.
 * https://github.com/facebook/create-react-app/issues/1023#issuecomment-265344421
 *
 * We also resolve them to make sure all tools using them work consistently.
 */
function setNodePath() {
    process.env.NODE_PATH = (process.env.NODE_PATH || '')
        .split(path.delimiter)
        .filter(folder => folder && !path.isAbsolute(folder))
        .map(paths_1.resolve)
        .join(path.delimiter);
}
function loadEnvVariables(currentEnvMode) {
    // https://github.com/bkeepers/dotenv#what-other-env-files-can-i-use
    const dotenvFiles = [
        `${paths_1.DOT_ENV_FILE_PATH}.${currentEnvMode}.local`,
        `${paths_1.DOT_ENV_FILE_PATH}.${currentEnvMode}`,
        // Don't include `.env.local` for `test` environment
        // since normally you expect tests to produce the same
        // results for everyone
        currentEnvMode !== 'test' && `${paths_1.DOT_ENV_FILE_PATH}.local`,
        paths_1.DOT_ENV_FILE_PATH,
    ].filter(Boolean);
    // Load environment variables from .env* files. Suppress warnings using silent
    // if this file is missing. dotenv will never modify any environment variables
    // that have already been set.  Variable expansion is supported in .env files.
    // https://github.com/motdotla/dotenv
    // https://github.com/motdotla/dotenv-expand
    dotenvFiles.forEach(async (dotenvPath) => {
        const withDotenvFile = paths_1.withFile(dotenvPath);
        withDotenvFile(() => {
            gluegun_1.print.info(`Loading Environment Variables File: ${dotenvPath}`);
            dotenv_expand_1.default(dotenv_1.default.config({ path: dotenvPath }));
        });
    });
}
function setEnvMode(env) {
    if (!ALLOWED_ENV.includes(env)) {
        throw new Error(`"${env}" mode is not supported.

      Available modes:

      - development
      - test
      - production

    `);
    }
    process.env.BABEL_ENV = env;
    process.env.NODE_ENV = env;
    gluegun_1.print.info(`Current Environment Mode: ${gluegun_1.print.colors.success(env)}`);
    return env;
}
function readEnv(envVariables) {
    const regex = /^STRAW_HAT_APP_/i;
    const extractVariable = (env, key) => {
        if (!regex.test(key)) {
            return env;
        }
        return {
            ...env,
            [key]: process.env[key],
        };
    };
    return Object.keys(process.env).reduce(extractVariable, envVariables);
}
exports.readEnv = readEnv;
const withYarnLock = paths_1.withFile(paths_1.YARN_LOCK_PATH);
function getPackageManager() {
    return withYarnLock(() => PackageManagers.yarn, () => PackageManagers.npm);
}
exports.getPackageManager = getPackageManager;
exports.isDevelopment = ramda_1.equals(Environments.development);
exports.isProduction = ramda_1.equals(Environments.production);
const asProcessEnv = (stringifiedEnvironment) => ({
    'process.env': stringifiedEnvironment,
});
const stringifyValue = (obj) => (env, key) => ({
    ...env,
    [key]: JSON.stringify(obj[key]),
});
function stringifyEnvironment(environment) {
    const initialState = {};
    const stringify = ramda_1.compose(asProcessEnv, ramda_1.reduce(stringifyValue(environment), initialState), ramda_1.keys);
    return stringify(environment);
}
exports.stringifyEnvironment = stringifyEnvironment;
const withStrawHatConfigFile = paths_1.withFile(paths_1.STRAW_HAT_CONFIG_PATH);
function strawHatConfigFactory(params) {
    return withStrawHatConfigFile(() => {
        gluegun_1.print.info(`Loading Straw Hat Config: ${paths_1.STRAW_HAT_CONFIG_PATH}`);
        const configFactory = require(paths_1.STRAW_HAT_CONFIG_PATH);
        return configFactory(params);
    }, () => ({}));
}
exports.strawHatConfigFactory = strawHatConfigFactory;
//# sourceMappingURL=environment.js.map