"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const gluegun_1 = require("gluegun");
const npm_packlist_1 = __importDefault(require("npm-packlist"));
const path = __importStar(require("path"));
const path_is_inside_1 = __importDefault(require("path-is-inside"));
const ramda_1 = require("ramda");
const environment_1 = require("./environment");
const paths_1 = require("./paths");
function normalizeTarballFileName(name) {
    return name.replace('/', '-').replace('@', '');
}
/**
 * Tarball file factory based on the package manager. Since `npm` and `yarn`
 * create the tarball file with different names.
 */
function getTarballFileName(packageManager, pkg) {
    switch (packageManager) {
        case environment_1.PackageManagers.npm:
            return normalizeTarballFileName(`${pkg.name}-${pkg.version}.tgz`);
        case environment_1.PackageManagers.yarn:
            return normalizeTarballFileName(`${pkg.name}-v${pkg.version}.tgz`);
        default:
            throw new Error(`${packageManager} is not supported`);
    }
}
exports.getTarballFileName = getTarballFileName;
const packageFile = paths_1.resolve('package.json');
exports.readPackage = ramda_1.compose(require, packageFile);
const removeFiles = ramda_1.omit(['files']);
async function createPackage(params) {
    const packageFilePath = packageFile(params.distDir);
    // Remove the pkg.files because we will copy over the files
    // that you want to include on the package
    const finalPkgConfig = removeFiles(params.pkg);
    await fs.writeJson(packageFilePath, finalPkgConfig, { spaces: 2 });
    gluegun_1.print.info(`File Created: ${packageFilePath}`);
}
async function getPackFiles(params) {
    const allFiles = await npm_packlist_1.default({ path: params.appPath });
    return allFiles.filter(file => {
        const filePath = paths_1.resolve(file)(params.appPath);
        // Do not copy package.json because we will create it
        if (file === 'package.json') {
            return false;
        }
        // TGZ packed package
        if (/\.tgz$/.test(file)) {
            return false;
        }
        // Do not copy anything inside the `dist` folder
        return !path_is_inside_1.default(filePath, params.distDir);
    });
}
/**
 * Returns the distribution path. It normalize the file location. If the file
 * is inside source code directory (`srcDir`) the file location is relative
 * to it on the distribution folder (`distDir`).
 */
function getDestinationPath(params) {
    const finalPath = params.fileAbsolutePath.includes(params.srcDir)
        ? path.relative(params.srcDir, params.fileAbsolutePath)
        : params.file;
    return paths_1.resolve(finalPath)(params.distDir);
}
async function copyFile(params) {
    const fileAbsolutePath = paths_1.resolve(params.file)(params.appPath);
    const destination = getDestinationPath({
        distDir: params.distDir,
        file: params.file,
        fileAbsolutePath,
        srcDir: params.srcDir,
    });
    await fs.copy(fileAbsolutePath, destination);
    gluegun_1.print.info(`File Copied: ${destination}`);
    return destination;
}
async function copyFiles(params) {
    const files = await getPackFiles(params);
    return Promise.all(files.map(async (file) => copyFile({
        appPath: params.appPath,
        distDir: params.distDir,
        file,
        srcDir: params.srcDir,
    })));
}
async function prepareDist(params) {
    await fs.ensureDir(params.distDir);
    gluegun_1.print.info(`Distribution Folder: ${params.distDir}`);
    await copyFiles(params);
    await createPackage(params);
    return params;
}
/**
 * Prepare the distribution directory for the package.
 *
 * @param appPath Application directory.
 */
async function preparePackage(appPath) {
    const pkg = exports.readPackage(appPath);
    return prepareDist({
        appPath,
        distDir: paths_1.DIST_PATH,
        pkg,
        srcDir: paths_1.SRC_PATH,
    });
}
exports.preparePackage = preparePackage;
//# sourceMappingURL=packager.js.map